@@ -353,6 +353,7 @@ void SolverRequester::updatePatches()
   // search twice: if there are none with restartSuggested(), retry on all
   // (in the first run, ignore_pkgmgmt == 0, in the second it is 1)
   bool any_marked = false;
+  bool dateLimit = ( _opts.date_limit != Date() );
   for (unsigned ignore_pkgmgmt = 0;
        !any_marked && ignore_pkgmgmt < 2; ++ignore_pkgmgmt)
   {
@@ -362,7 +363,22 @@ void SolverRequester::updatePatches()
       PackageSpec patch;
       patch.orig_str = (*it)->name();
       patch.parsed_cap = Capability((*it)->name());
-      if (installPatch(patch, (*it)->candidateObj(), ignore_pkgmgmt))
+
+      // bnc#919709: a date limit must ignore newer patch candidates
+      zypp::PoolItem candidateObj( (*it)->candidateObj() );
+      if ( dateLimit && asKind<Patch>(candidateObj)->timestamp() > _opts.date_limit )
+      {
+	for_( iit, (*it)->availableBegin(), (*it)->availableEnd() )
+	{
+	  if ( asKind<Patch>(*iit)->timestamp() <= _opts.date_limit )
+	  {
+	    candidateObj = *iit;
+	    break;
+	  }
+	}
+      }
+
+      if (installPatch(patch, candidateObj, ignore_pkgmgmt))
         any_marked = true;
     }
 
@@ -495,7 +511,7 @@ void SolverRequester::updateTo(
 
 
   // ******* request ********
-
+  bool action = true;
   if (!identical(installed, selected) || _opts.force)
   {
     if (_opts.best_effort)
@@ -522,19 +538,30 @@ void SolverRequester::updateTo(
       setToInstall(selected);
       MIL << *s << " update: setting " << selected << " to install (arch change request)" << endl;
     }
+    else if ( selected->edition() == installed->edition()
+	    && !pkg.repo_alias.empty() /*userselected repo*/ )
+    {
+      // set 'candidate' for installation
+      setToInstall(selected);
+      MIL << *s << " update: setting " << selected << " to install (repo change request)" << endl;
+    }
     else if (_opts.force || _opts.oldpackage)
     {
       // set 'candidate' for installation
       setToInstall(selected);
       MIL << *s << " update: forced setting " << selected << " to install" << endl;
     }
+    else
+      action = false;
   }
+  else
+    action = false;
 
 
   // ******* report ********
 
   // the candidate is already installed
-  if (identical(installed, selected))
+  if (identical(installed, selected) || (!action && installed->edition() == selected->edition()))
   {
     if (_opts.force)
       return;
@@ -562,7 +589,7 @@ void SolverRequester::updateTo(
     }
 
     // the highest version is already there
-    if (identical(installed, highest) || highest->edition() < installed->edition())
+    if (identical(installed, highest) || highest->edition() <= installed->edition())
       addFeedback(Feedback::NO_UPD_CANDIDATE, pkg, selected, installed);
   }
   else if (installed->edition() > selected->edition())
@@ -575,6 +602,7 @@ void SolverRequester::updateTo(
         " Will not downgrade unless --oldpackage is used" << endl;
   }
 
+
   // there is higher version available than the selected candidate
   // this can happen because of repo priorities, locks, vendor lock, and
   // because of conditions given on comm. line: repos/arch/version (bnc #522223)
