@@ -13,6 +13,7 @@
 #include <zypp/ZYppFactory.h>
 #include <zypp/base/LogTools.h>
 #include <zypp/base/Measure.h>
+#include <zypp/base/DtorReset.h>
 #include <zypp/ResPool.h>
 #include <zypp/Patch.h>
 #include <zypp/Package.h>
@@ -385,20 +386,48 @@ namespace
   }
 } // namespace
 ///////////////////////////////////////////////////////////////////
-void Summary::writeResolvableList(ostream & out, const ResPairSet & resolvables)
+void Summary::writeResolvableList(ostream & out, const ResPairSet & resolvables, ansi::Color color)
 {
-
   if ((_viewop & DETAILS) == 0)
   {
+    static const ColorString quoteCh( "\"", ColorContext::HIGHLIGHT );
+
+    zypp::TriBool pkglistHighlight = Zypper::instance()->config().color_pkglistHighlight;
+    ansi::Color   pkglistHighlightAttribute = Zypper::instance()->config().color_pkglistHighlightAttribute;
+    char firstCh = 0;
+
     ostringstream s;
     for (ResPairSet::const_iterator resit = resolvables.begin();
         resit != resolvables.end(); ++resit)
     {
       // name
-      s << ResPair2Name( *resit );
+      const std::string & name( ResPair2Name( *resit ) );
+      if ( name.empty() )
+	continue;
+
+      // quote names with spaces
+      bool quote = name.find_first_of( " " ) != std::string::npos;
+
+      // quote?
+      if ( quote ) s << quoteCh;
+
+      // highlight 1st char?
+      if ( pkglistHighlight || ( indeterminate(pkglistHighlight) && name[0] != firstCh ) )
+      {
+	s << ( color << pkglistHighlightAttribute << name[0] ) << name.c_str()+1;
+	if ( indeterminate(pkglistHighlight) )
+	   firstCh = name[0];
+      }
+      else
+      {
+	s << name;
+      }
+
+      // quote?
+      if ( quote ) s << quoteCh;
 
       // version (if multiple versions are present)
-      if (!(_viewop & SHOW_VERSION) && multi_installed.find(resit->second->name()) != multi_installed.end())
+      if (multi_installed.find(resit->second->name()) != multi_installed.end())
       {
         if (resit->first && resit->first->edition() != resit->second->edition())
           s << "-" << resit->first->edition().asString()
@@ -508,9 +537,9 @@ void Summary::writeNewlyInstalled(ostream & out)
         "The following %d applications are going to be installed:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::POSITIVE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::POSITIVE);
   }
 }
 
@@ -518,7 +547,7 @@ void Summary::writeNewlyInstalled(ostream & out)
 
 void Summary::writeRemoved(ostream & out)
 {
-  ViewOptions vop = _viewop;
+  DtorReset guard( _viewop );
   unsetViewOption(SHOW_REPO); // never show repo here, it's always @System
   for_(it, _toremove.begin(), _toremove.end())
   {
@@ -549,11 +578,10 @@ void Summary::writeRemoved(ostream & out)
         "The following %d applications are going to be REMOVED:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::NEGATIVE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::NEGATIVE);
   }
-  _viewop = vop;
 }
 
 // --------------------------------------------------------------------------
@@ -589,9 +617,9 @@ void Summary::writeUpgraded(ostream & out)
         "The following %d applications are going to be upgraded:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::POSITIVE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::POSITIVE);
   }
 }
 
@@ -628,9 +656,9 @@ void Summary::writeDowngraded(ostream & out)
         "The following %d applications are going to be downgraded:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::NEGATIVE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::NEGATIVE);
   }
 }
 
@@ -662,14 +690,22 @@ void Summary::writeReinstalled(ostream & out)
         "The following %d products are going to be reinstalled:",
         it->second.size());
     else if (it->first == ResKind::application)
+    {
+      // Suppress 'reinstalled' message for applications.
+      // As applications don't have an edition, they are not updated
+      // if the package updates. Technically every app-package update
+      // is an 'application reinstall', because the providing package
+      // changes.
+      return;
       label = _PL(
         "The following application is going to be reinstalled:",
         "The following %d applications are going to be reinstalled:",
         it->second.size());
+    }
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::CHANGE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::CHANGE);
   }
 }
 
@@ -833,9 +869,9 @@ void Summary::writeRecommended(ostream & out)
         "The following %d recommended applications were automatically selected:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::POSITIVE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::POSITIVE);
   }
 
   for_(it, _noinstrec.begin(), _noinstrec.end())
@@ -877,8 +913,9 @@ void Summary::writeRecommended(ostream & out)
 		     "The following %d packages are recommended, but will not be installed (only required packages will be installed):",
 		     it->second.size() );
 	label = str::form( label.c_str(), it->second.size() );
-	out << endl << label << endl;
-	writeResolvableList(out, notRequired);
+
+	out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+	writeResolvableList(out, notRequired, ColorContext::HIGHLIGHT);
       }
       else
       {
@@ -887,9 +924,10 @@ void Summary::writeRecommended(ostream & out)
 	  label = _PL( "The following package is recommended, but will not be installed because it's unwanted (was manually removed before):",
 		       "The following %d packages are recommended, but will not be installed because they are unwanted (were manually removed before):",
 		       it->second.size() );
-	label = str::form( label.c_str(), it->second.size() );
-	out << endl << label << endl;
-	  writeResolvableList(out, softLocked);
+	  label = str::form( label.c_str(), it->second.size() );
+
+	  out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+	  writeResolvableList(out, softLocked, ColorContext::HIGHLIGHT);
         }
         if ( !conflicts.empty() )
         {
@@ -897,8 +935,9 @@ void Summary::writeRecommended(ostream & out)
 		       "The following %d packages are recommended, but will not be installed due to conflicts or dependency issues:",
 		       it->second.size() );
 	  label = str::form( label.c_str(), it->second.size() );
-          out << endl << label << endl;
-          writeResolvableList(out, conflicts);
+
+	  out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+          writeResolvableList(out, conflicts, ColorContext::HIGHLIGHT);
         }
       }
     }
@@ -921,8 +960,9 @@ void Summary::writeRecommended(ostream & out)
 		     "The following %d applications are recommended, but will not be installed:",
 		     it->second.size() );
       label = str::form( label.c_str(), it->second.size() );
-      out << endl << label << endl;
-      writeResolvableList(out, it->second);
+
+      out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+      writeResolvableList(out, it->second, ColorContext::HIGHLIGHT);
     }
   }
 
@@ -980,9 +1020,9 @@ void Summary::writeSuggested(ostream & out)
         "The following %d applications are suggested, but will not be installed:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::HIGHLIGHT);
   }
 }
 
@@ -990,7 +1030,7 @@ void Summary::writeSuggested(ostream & out)
 
 void Summary::writeChangedArch(ostream & out)
 {
-  ViewOptions vop = _viewop;
+  DtorReset guard( _viewop );
   setViewOption(SHOW_ARCH); // always show arch here
   for_(it, _tochangearch.begin(), _tochangearch.end())
   {
@@ -1021,18 +1061,17 @@ void Summary::writeChangedArch(ostream & out)
         "The following %d applications are going to change architecture:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::CHANGE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::CHANGE);
   }
-  _viewop = vop;
 }
 
 // --------------------------------------------------------------------------
 
 void Summary::writeChangedVendor(ostream & out)
 {
-  ViewOptions vop = _viewop;
+  DtorReset guard( _viewop );
   setViewOption(SHOW_VENDOR); // always show vendor here
   for_(it, _tochangevendor.begin(), _tochangevendor.end())
   {
@@ -1063,11 +1102,10 @@ void Summary::writeChangedVendor(ostream & out)
         "The following %d applications are going to change vendor:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::CHANGE << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::CHANGE);
   }
-  _viewop = vop;
 }
 
 // --------------------------------------------------------------------------
@@ -1084,9 +1122,9 @@ void Summary::writeUnsupported(ostream & out)
         "The following %d packages are not supported by their vendor:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::HIGHLIGHT);
   }
 }
 
@@ -1104,9 +1142,9 @@ void Summary::writeNeedACC(ostream & out)
         "The following %d packages need additional customer contract to get support:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::HIGHLIGHT);
   }
 }
 
@@ -1132,9 +1170,53 @@ void Summary::writeNotUpdated(std::ostream & out)
         "The following %d application updates will NOT be installed:",
         it->second.size());
     label = str::form( label.c_str(), it->second.size() );
-    out << endl << label << endl;
 
-    writeResolvableList(out, it->second);
+    out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+    writeResolvableList(out, it->second, ColorContext::HIGHLIGHT);
+  }
+}
+
+void Summary::writeLocked(std::ostream & out)
+{
+  return;
+  ResPairSet instlocks;	// locked + installed
+  ResPairSet avidents;	// avaialble locked
+  ResPoolProxy selPool( ResPool::instance().proxy() );
+  for_( it, selPool.begin(), selPool.end() )
+  {
+    if ( (*it)->locked() )
+    {
+      if ( (*it)->hasInstalledObj() )
+       for_( iit, (*it)->installedBegin(), (*it)->installedEnd() )
+         instlocks.insert( ResPair( NULL, *iit ) );
+      else
+       ;//avidents.insert( ResPair( NULL, (*it)->theObj() ) );
+    }
+  }
+  if ( ! ( instlocks.empty() && avidents.empty() ) )
+  {
+    string label = _PL(
+      "The following item is locked and will not be changed by any action:",
+      "The following %d items are locked and will not be changed by any action:",
+      ( instlocks.size() + avidents.size() )
+    );
+    label = str::form( label.c_str(), instlocks.size() + avidents.size() );
+    out << endl << ( ColorContext::HIGHLIGHT << label ) << endl;
+
+    if ( ! avidents.empty() )
+    {
+      DtorReset guard( _viewop );
+      _viewop = DEFAULT;	// always as plain name list
+      // translators: used as 'tag:' (i.e. followed by ':')
+      out << " " << _("Available") << ':' << endl;
+      writeResolvableList( out, avidents, ColorContext::HIGHLIGHT );
+    }
+    if ( ! instlocks.empty() )
+    {
+      // translators: used as 'tag:' (i.e. followed by ':')
+      out << " " << _("Installed") << ':' << endl;
+      writeResolvableList( out, instlocks, ColorContext::HIGHLIGHT );
+    }
   }
 }
 
@@ -1188,7 +1270,7 @@ void Summary::writePackageCounts(ostream & out)
   i = _toupgrade.find(ResKind::package);
   if (i != _toupgrade.end() && (count = i->second.size()) )
   {
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_HIGHLIGHT);
+    s << ( ColorContext::POSITIVE << count ) << " ";
     // translators: this text will be preceded by a number e.g. "5 packages to ..."
     s << _PL("package to upgrade", "packages to upgrade", count);
     gotcha = true;
@@ -1198,7 +1280,7 @@ void Summary::writePackageCounts(ostream & out)
   {
     if (gotcha)
       s << ", ";
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_HIGHLIGHT);
+    s << ( ColorContext::NEGATIVE << count ) << " ";
     if (gotcha)
       // translators: this text will be preceded by a number e.g. "5 to ..."
       s << _PL("to downgrade", "to downgrade", count);
@@ -1212,7 +1294,7 @@ void Summary::writePackageCounts(ostream & out)
   {
     if (gotcha)
       s << ", ";
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_HIGHLIGHT);
+    s << ( ColorContext::POSITIVE << count ) << " ";
     if (gotcha)
       // translators: this text will be preceded by a number e.g. "5 new"
       s << _PL("new", "new", count);
@@ -1226,7 +1308,7 @@ void Summary::writePackageCounts(ostream & out)
   {
     if (gotcha)
       s << ", ";
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_HIGHLIGHT);
+    s << ( ColorContext::CHANGE << count ) << " ";
     if (gotcha)
       // translators: this text will be preceded by a number e.g. "5 to ..."
       s << _PL("to reinstall", "to reinstall", count);
@@ -1240,7 +1322,7 @@ void Summary::writePackageCounts(ostream & out)
   {
     if (gotcha)
       s << ", ";
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_NEGATIVE);
+    s << ( ColorContext::NEGATIVE << count ) << " ";
     if (gotcha)
       // translators: this text will be preceded by a number e.g. "5 to ..."
       s << _PL("to remove", "to remove", count);
@@ -1254,7 +1336,7 @@ void Summary::writePackageCounts(ostream & out)
   {
     if (gotcha)
       s << ", ";
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_NEGATIVE);
+    s << ( ColorContext::CHANGE << count ) << " ";
     if (gotcha)
       // translators: this text will be preceded by a number e.g. "5 to ..."
       s << _PL("to change vendor", " to change vendor", count);
@@ -1268,7 +1350,7 @@ void Summary::writePackageCounts(ostream & out)
   {
     if (gotcha)
       s << ", ";
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_HIGHLIGHT);
+    s << ( ColorContext::CHANGE << count ) << " ";
     if (gotcha)
       // translators: this text will be preceded by a number e.g. "5 to ..."
       s << _PL("to change arch", "to change arch", count);
@@ -1282,7 +1364,7 @@ void Summary::writePackageCounts(ostream & out)
   {
     if (gotcha)
       s << ", ";
-    fprint_color(s, str::form("%d ", count), COLOR_CONTEXT_HIGHLIGHT);
+    s << ( ColorContext::POSITIVE << count ) << " ";
     if (gotcha)
       // translators: this text will be preceded by a number e.g. "5 new"
       s << _PL("source package", "source packages", count);
@@ -1311,6 +1393,7 @@ void Summary::dumpTo(ostream & out)
 
   _wrap_width = get_screen_width();
 
+  writeLocked(out);
   if (_viewop & SHOW_NOT_UPDATED)
     writeNotUpdated(out);
   writeNewlyInstalled(out);
