@@ -28,6 +28,11 @@ using namespace zypp;
 class Table;
 class Zypper;
 
+#define OSD ColorStream( std::cout, ColorContext::OSDEBUG )
+
+inline char * asYesNo( bool val_r )
+{ return val_r ? _("Yes") : _("No"); }
+
 ///////////////////////////////////////////////////////////////////
 namespace out
 {
@@ -194,8 +199,6 @@ struct TermLine
 
   zypp::str::Str lhs;				//< left side
   zypp::str::Str rhs;				//< right side
-  zypp::DefaultIntegral<unsigned,0> lhidden;	//< size of embedded esc sequences
-  zypp::DefaultIntegral<unsigned,0> rhidden;	//< size of embedded esc sequences
 
 
   /** Return plain line made of lhs + rhs */
