@@ -16,13 +16,17 @@
 #include <zypp/media/MediaManager.h>
 #include <zypp/misc/CheckAccessDeleted.h>
 #include <zypp/ExternalProgram.h>
+#include "zypp/parser/ProductFileReader.h"
 
+#include <zypp/ZYpp.h>
+#include <zypp/Target.h>
 #include <zypp/PoolItem.h>
 #include <zypp/Product.h>
 #include <zypp/Pattern.h>
 
 #include "main.h"
 #include "Zypper.h"
+#include "repos.h"
 #include "Table.h"             // for process list in suggest_restart_services
 
 #include "utils/misc.h"
@@ -31,6 +35,21 @@
 using namespace std;
 using namespace zypp;
 
+extern ZYpp::Ptr God;
+
+bool runningOnEnterprise()
+{
+  bool ret = false;
+  if ( Target_Ptr target = God->getTarget() )
+    if ( Product::constPtr platform = target->baseProduct() )
+    {
+      static const CpeId enterprise( "cpe:/o:suse:sle%02" );
+      if ( compare( platform->cpeId(), enterprise, SetRelation::subset ) )
+	ret = true;
+    }
+  return ret;
+}
+
 // ----------------------------------------------------------------------------
 
 bool is_changeable_media(const zypp::Url & url)
@@ -209,6 +228,29 @@ Url make_url (const string & url_s)
 // http://gitorious.org/opensuse/build-service/blobs/master/src/webui/app/controllers/application_controller.rb
 #define OBS_PROJECT_NAME_RX "[" ALNUM "][-+" ALNUM "\\.:]+"
 
+///////////////////////////////////////////////////////////////////
+namespace {
+  parser::ProductFileData baseproductdata( const Pathname & root_r )
+  {
+    parser::ProductFileData ret;
+    PathInfo baseproduct( Pathname::assertprefix( root_r, "/etc/products.d/baseproduct" ) );
+
+    if ( baseproduct.isFile() )
+    {
+      try
+      {
+	ret = parser::ProductFileReader::scanFile( baseproduct.path() );
+      }
+      catch ( const Exception & excpt )
+      {
+	ZYPP_CAUGHT( excpt );
+      }
+    }
+    return ret;
+  }
+
+} // namespace
+///////////////////////////////////////////////////////////////////
 
 Url make_obs_url( const string & obsuri, const Url & base_url, const string & default_platform )
 {
@@ -247,7 +289,43 @@ Url make_obs_url( const string & obsuri, const Url & base_url, const string & de
 
     Pathname path( ret.getPathName() );
     path /= str::gsub( project, ":", ":/" );
-    path /= ( platform.empty() ? default_platform : platform );
+
+    if ( platform.empty() )
+    {
+      Zypper & zypper( *Zypper::instance() );
+
+      if ( default_platform.empty() )
+      {
+	// Try to guess platform from baseproduct....
+	const parser::ProductFileData & pdata( baseproductdata( zypper.globalOpts().root_dir ) );
+	if ( pdata.empty() )
+	{
+	  // Guess failed:
+			     // translators: don't translate '<platform>'
+	  zypper.out().error(_("Unable to guess a value for <platform>."),
+			     _("Please use obs://<project>/<platform>") );
+	  zypper.out().info(str::form(_("Example: %s"), "obs://server:http/openSUSE_11.3"));
+	  return Url();	// FAIL!
+	}
+
+	platform = pdata.name().asString();
+	if ( platform == "openSUSE" && str::containsCI( pdata.summary(), "Tumbleweed" ) )
+	  platform += "_Tumbleweed";
+	else
+	  platform += "_$releasever";
+
+	zypper.out().info( "Guessed: platform = " + platform );
+	path /= platform;
+      }
+      else
+      {
+	zypper.out().info( "zypper.conf: obs.platform = " + default_platform );
+	path /= default_platform;
+      }
+    }
+    else
+      path /= platform;
+
     ret.setPathName( path.asString() );
 
     return ret;
@@ -359,6 +437,7 @@ string asXML(const Product & p, bool is_installed)
     } );
 
     dumpAsXmlOn( *parent, p.endOfLife(), "endoflife" );
+    dumpAsXmlOn( *parent, p.registerFlavor(), "registerflavor" );
     {
       const std::string & text( p.description() );
       if ( ! text.empty() )
@@ -423,7 +502,7 @@ void list_processes_using_deleted_files(Zypper & zypper)
   // process user ID
   th << _("UID");
   // process login name
-  th << _("Login");
+  th << _("User");
   // process command name
   th << _("Command");
   // "/etc/init.d/ script that might be used to restart the command (guessed)
